﻿using FreeSql;
using FreeSql.DataAnnotations;
using Newtonsoft.Json;
using System.Collections.Generic;

namespace ojbk.Entities
{
    /// <summary>
    /// 后台路由，一级=菜单
    /// </summary>
    public partial class Menu : BaseEntity<Menu, int>
    {
        static Menu()
        {
            #region 初始化菜单
            if (Orm.Select<Menu>().Any() == false)
            {
                var adds = new[]
                {
                    new Menu
                    {
                        Name = "文档",
                        Sort = 1,
                        Extdata = JsonConvert.SerializeObject(new { icon = "ios-book", href = "https://github.com/2881099/ojbk" }),
                    },
                    new Menu
                    {
                        Name = "QQ群",
                        Sort = 2,
                        Extdata = JsonConvert.SerializeObject(new { icon = "_qq", path = "/join_qqgroup", name = "join_qqgroup", component = "@/view/join_qqgroup-page.vue" }),
                    },

                    new Menu
                    {
                        Name = "系统管理",
                        Sort = 3,
                        Extdata = JsonConvert.SerializeObject(new { icon = "logo-buffer", path = "/sysadmin", name = "sysadmin" }),

                        Childs = new List<Menu>( new []
                        {
                            new Menu
                            {
                                Name = "用户管理",
                                Sort = 1,
                                Extdata = JsonConvert.SerializeObject(new { icon = "org_tree_page", path = "/authuser", name = "sysadmin_authuser", component = "@/view/sysadmin/authuser-page.vue" }),
                            },
                            new Menu
                            {
                                Name = "角色管理",
                                Sort = 2,
                                Extdata = JsonConvert.SerializeObject(new { icon = "org_tree_page", path = "/authrole", name = "sysadmin_authrole", component = "@/view/sysadmin/authrole-page.vue" }),
                            },
                            new Menu
                            {
                                Name = "菜单管理",
                                Sort = 3,
                                Extdata = JsonConvert.SerializeObject(new { icon = "org_tree_page", path = "/admroute", name = "sysadmin_admroute", component = "@/view/sysadmin/admroute-page.vue" }),
                            },
                            new Menu
                            {
                                Name = "权限管理",
                                Sort = 4,
                                Extdata = JsonConvert.SerializeObject(new { icon = "org_tree_page", path = "/auth", name = "sysadmin_auth", component = "@/view/sysadmin/auth-page.vue" }),
                            },
                        })
                    },

                    new Menu
                    {
                        Name = "组织机构",
                        Sort = 4,
                        Extdata = JsonConvert.SerializeObject(new { icon = "logo-buffer", path = "/organization", name = "organization" }),

                        Childs = new List<Menu>( new []
                        {
                            new Menu
                            {
                                Name = "部门管理",
                                Sort = 1,
                                Extdata = JsonConvert.SerializeObject(new { icon = "org_tree_page", path = "/department", name = "organization_department", component = "@/view/organization/department-page.vue" }),
                            },
                            new Menu
                            {
                                Name = "岗位管理",
                                Sort = 2,
                                Extdata = JsonConvert.SerializeObject(new { icon = "org_tree_page", path = "/post", name = "organization_post", component = "@/view/organization/post-page.vue" }),
                            },
                            new Menu
                            {
                                Name = "员工管理",
                                Sort = 3,
                                Extdata = JsonConvert.SerializeObject(new { icon = "org_tree_page", path = "/person", name = "organization_person", component = "@/view/organization/person-page.vue" }),
                            }
                        })
                    },

                };
                Orm.GetRepository<Menu>().Insert(adds);
            }
            #endregion
        }

        [Navigate(nameof(ParentId))]
        public List<Menu> Childs { get; set; }

        /// <summary>
        /// 父节点
        /// </summary>
        public int ParentId { get; set; }
        [Navigate(nameof(ParentId))]
        public Menu Parent { get; set; }

        /// <summary>
        /// 标题
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// MVC 路由名称
        /// </summary>
        public string Controller { get; set; }

        /// <summary>
        /// 前端数据
        /// </summary>
        public string Extdata { get; set; }
        
        /// <summary>
        /// 备注
        /// </summary>
        public string Remark { get; set; }
    }

    #region ManyToMany Role
    partial class Role
    {
        [Navigate(ManyToMany = typeof(AuthRoleAdmRoute))]
        public List<Menu> AdmRoutes { get; set; }

        public class AuthRoleAdmRoute : BaseEntity<AuthRoleAdmRoute>
        {
            public int RoleId { get; set; }
            public Role Role { get; set; }

            public int AdmRouteId { get; set; }
            public Menu AdmRoute { get; set; }
        }
    }
    partial class Menu
    {
        [Navigate(ManyToMany = typeof(Role.AuthRoleAdmRoute))]
        public List<Role> Roles { get; set; }
    }
    #endregion
}
