﻿using FreeSql;
using FreeSql.DataAnnotations;
using Newtonsoft.Json;
using System;
using System.Linq;

namespace ojbk.Entities
{
    /// <summary>
    /// 登陆账户
    /// </summary>
    [Index("uk_{tablename}_username", nameof(Username))]
    public partial class User : BaseEntity<User, int>
    {
        static User()
        {
            if (User.Select.Any(a => a.IsAdmin) == false)
            {
                new User
                {
                    IsAdmin = true,
                    Password = Util.Md5("admin"),
                    Status = AuthUserStatus.正常,
                    Username = "admin"
                }.Insert();
            }
        }

        /// <summary>
        /// 登陆用户，查询后台路由表
        /// </summary>
        [JsonIgnore]
        public ISelect<Menu> SelectAdmRoute => this.IsAdmin ? 
            Menu.Select :
            Menu.Select.Where(a => a.Roles.Any(role => role.RoleUsers.Any(ur => ur.UserId == this.Id)));

        /// <summary>
        /// 员工id
        /// </summary>
        public int EmployeeId { get; set; }
        /// <summary>
        /// 员工
        /// </summary>
        [Navigate(nameof(EmployeeId))]
        public Employee Employee { get; set; }

        /// <summary>
        /// 用户名
        /// </summary>
        public string Username { get; set; }

        /// <summary>
        /// 密码
        /// </summary>
        public string Password { get; set; }

        /// <summary>
        /// 最后登录时间
        /// </summary>
        public DateTime LoginTime { get; set; }
        /// <summary>
        /// 最后登录IP
        /// </summary>
        public string LoginIp { get; set; }

        /// <summary>
        /// 状态
        /// </summary>
        public AuthUserStatus Status { get; set; }
        /// <summary>
        /// 是否为系统管理员
        /// </summary>
        public bool IsAdmin { get; set; }
    }

    /// <summary>
    /// 账户状态
    /// </summary>
    public enum AuthUserStatus { 正常, 禁用 }
}
